@@ -16,18 +16,29 @@
 
 #include "qquickwindow_p.h"
 #include <QtQml/private/qv4persistent_p.h>
+#include "qquickwindowcontainer_p.h"
 
 QT_BEGIN_NAMESPACE
 
-class Q_QUICK_PRIVATE_EXPORT QQuickWindowQmlImplPrivate : public QQuickWindowPrivate
+class Q_QUICK_EXPORT QQuickWindowQmlImplPrivate : public QQuickWindowPrivate
 {
 public:
     QQuickWindowQmlImplPrivate();
 
+    bool componentComplete = true;
+
     bool visible = false;
     bool visibleExplicitlySet = false;
     QQuickWindow::Visibility visibility = QQuickWindow::AutomaticVisibility;
+    bool visibilityExplicitlySet = false;
+
     QV4::PersistentValue rootItemMarker;
+
+    QMetaObject::Connection itemParentWindowChangeListener;
+
+    QObject *visualParent = nullptr;
+    QPointer<QQuickWindowContainer> windowContainer;
+    qreal z = 0.0;
 };
 
 QT_END_NAMESPACE
