@@ -1,11 +1,13 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include <QtCore/qglobal.h>
-
 #ifndef QTQUICKCONTROLS2IMPLEXPORTS_H
 #define QTQUICKCONTROLS2IMPLEXPORTS_H
 
+#include <QtCore/qcompilerdetection.h>
+#include <QtCore/qtconfigmacros.h> // Q_QUICKCONTROLS2IMPL_EXPORT
+#include <QtCore/qtdeprecationmarkers.h> // QT_IF_DEPRECATED_SINCE
+
 #if defined(QT_SHARED) || !defined(QT_STATIC)
 #  if defined(QT_BUILD_QUICKCONTROLS2IMPL_LIB)
 #    define Q_QUICKCONTROLS2IMPL_EXPORT Q_DECL_EXPORT
@@ -17,22 +19,22 @@
 #endif
 
 #if !defined(QT_BUILD_QUICKCONTROLS2IMPL_LIB) && !defined(QT_STATIC)
-/* outside library → inline decl + defi */
+/* outside library -> inline decl + defi */
 /* static builds treat everything as part of the library, so they never inline */
 #  define QT_QUICKCONTROLS2IMPL_INLINE_SINCE(major, minor) inline
 #  define QT_QUICKCONTROLS2IMPL_INLINE_IMPL_SINCE(major, minor) 1
 #elif defined(QT_QUICKCONTROLS2IMPL_BUILD_REMOVED_API)
 /* inside library, inside removed_api.cpp:
- * keep deprecated API → non-inline decl;
- * remove deprecated API → inline decl;
+ * keep deprecated API -> non-inline decl;
+ * remove deprecated API -> inline decl;
  * definition is always available */
 #  define QT_QUICKCONTROLS2IMPL_INLINE_SINCE(major, minor) \
     QT_IF_DEPRECATED_SINCE(major, minor, inline, /* not inline */)
 #  define QT_QUICKCONTROLS2IMPL_INLINE_IMPL_SINCE(major, minor) 1
 #else
 /* inside library, outside removed_api.cpp:
- * keep deprecated API → non-inline decl, no defi;
- * remove deprecated API → inline decl, defi */
+ * keep deprecated API -> non-inline decl, no defi;
+ * remove deprecated API -> inline decl, defi */
 #  define QT_QUICKCONTROLS2IMPL_INLINE_SINCE(major, minor) \
     QT_IF_DEPRECATED_SINCE(major, minor, inline, /* not inline */)
 #  define QT_QUICKCONTROLS2IMPL_INLINE_IMPL_SINCE(major, minor) \
